"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CliRenderer = void 0;
const deps_1 = require("./deps");
const utils_1 = require("./utils");
const SEP = ' ';
const EOL = '\n';
const LIST_CHAR = '྿';
const NOT_EMPTY = flag => !!flag;
const block = text => EOL + text + EOL;
const mapLines = (text, mapper) => text.split(EOL).map(mapper).join(EOL) + EOL;
class CliRenderer extends deps_1.Renderer {
    constructor(opts) {
        super();
        this.opts = opts;
    }
    // INLINE
    checkbox(checked) {
        const { cbStyle, cbUncheckedChar, cbCheckedChar } = this.opts;
        return cbStyle(checked ? cbCheckedChar : cbUncheckedChar) + SEP;
    }
    strong(text) {
        return this.opts.strongStyle(text);
    }
    em(text) {
        return this.opts.emStyle(text);
    }
    codespan(code) {
        return this.em(this.opts.codeStyle(code));
    }
    br() {
        return EOL;
    }
    del(text) {
        return this.opts.delStyle(text);
    }
    link(href, title, text) {
        // todo need to be refactor
        const { linkStyle } = this.opts;
        // if (supportsHyperlinks.stout || true) {
        //     return hyperLinker(linkStyle(text), href);
        // }
        return href === text ? linkStyle(href) : `${text}(${linkStyle(href)})`;
    }
    image(href, title, text) {
        // no image support in terminal
        return '🌆';
    }
    text(text) {
        return utils_1.textify()(text);
        // return textify(this.wrapper)(text);
    }
    // BLOCK
    code(code, infostring, escaped) {
        const { lineLength, codeStyle, codeInfoStyle } = this.opts;
        const mapper = line => SEP + codeStyle((SEP + line).padEnd(lineLength - 2, SEP));
        const rendered = mapLines(block(code.trim()), mapper);
        return infostring
            ? deps_1.highlight(rendered, { language: infostring }) +
                SEP +
                codeInfoStyle(SEP + infostring + SEP) +
                EOL
            : rendered;
    }
    blockquote(quote) {
        const { quotePadding, quoteChar, quoteStyle } = this.opts;
        const mapper = line => quoteStyle(SEP.repeat(quotePadding) + quoteChar + SEP + line);
        return block(mapLines(quote.trim(), mapper));
    }
    heading(text, level, raw, slugger) {
        const { headingLevels } = this.opts;
        const style = deps_1.chalk.hex(headingLevels[level - 1]);
        return utils_1.pipe(style, this.wrapper, block)(text);
    }
    hr() {
        const { lineLength, hrStyle, hrChar } = this.opts;
        return block(SEP + hrStyle(hrChar.repeat(lineLength - 2)) + SEP);
    }
    paragraph(text) {
        return block(this.wrapper(text));
    }
    // COMPOUND
    html(html) {
        return block(deps_1.chalk.redBright('HTML need to be implemented'));
    }
    list(body, ordered, start) {
        // todo consider use data as table
        const { listChar, listStyle } = this.opts;
        const mapper = ordered
            ? line => SEP + line.replace(LIST_CHAR, listStyle(start++))
            : line => SEP + line.replace(LIST_CHAR, listStyle(listChar));
        return EOL + body.split(EOL).filter(NOT_EMPTY).map(mapper).join(EOL) + EOL;
    }
    listitem(text) {
        const mapper = (line, index) => index === 0 ? `${LIST_CHAR} ${line}` : `  ${line}`;
        return text.split(EOL).filter(NOT_EMPTY).map(mapper).join(EOL).trim() + EOL;
    }
    table(header, body) {
        const { lineLength, tableWordWrap: wordWrap } = this.opts;
        const head = utils_1.fromArray(header);
        const rows = utils_1.fromNestedArray(body);
        // create table with no columns restrictions
        let table = new deps_1.Table({ head, wordWrap });
        table.push(...rows);
        const output = table.toString();
        const length = output.search(EOL);
        if (length <= lineLength) {
            return output + EOL;
        }
        // re-create table with normalized columns
        const width = Math.ceil(lineLength / head.length);
        table = new deps_1.Table({ head, wordWrap, colWidths: head.map(() => width) });
        table.push(...rows);
        return table.toString() + EOL;
    }
    tablerow(content) {
        return utils_1.asArray(content);
    }
    tablecell(content, flags) {
        return utils_1.asObject(content);
    }
    // helpers
    get wrapper() {
        const { lineLength: width, indent } = this.opts;
        return utils_1.wrapper({ width, indent });
    }
}
exports.CliRenderer = CliRenderer;
